package io.gitlab.jfronny.inceptum.gtk

import io.gitlab.jfronny.commons.StringFormatter
import io.gitlab.jfronny.inceptum.common.Utils
import io.gitlab.jfronny.inceptum.gtk.util.markup
import io.gitlab.jfronny.inceptum.gtk.window.dialog.MicrosoftLoginDialog
import io.gitlab.jfronny.inceptum.gtk.window.dialog.StringInputDialog
import io.gitlab.jfronny.inceptum.launcher.LauncherEnv.EnvBackend
import io.gitlab.jfronny.inceptum.launcher.api.account.MicrosoftAccount
import org.gnome.gtk.*
import java.util.function.Consumer

object GtkEnvBackend : EnvBackend {
    @JvmField
    var dialogParent: Window? = null

    override fun showError(message: String, title: String) {
        Utils.LOGGER.error(message)
        simpleDialog(message, title, MessageType.ERROR, ButtonsType.CLOSE, null, null)
    }

    override fun showError(message: String, t: Throwable) {
        simpleDialog(StringFormatter.toString(t), message, MessageType.ERROR, ButtonsType.CLOSE, null, null)
    }

    override fun showInfo(message: String, title: String) {
        Utils.LOGGER.info(message)
        simpleDialog(message, title, MessageType.INFO, ButtonsType.CLOSE, null, null)
    }

    override fun showOkCancel(message: String, title: String, ok: Runnable, cancel: Runnable, defaultCancel: Boolean) {
        Utils.LOGGER.info(message)
        simpleDialog(message, title, MessageType.QUESTION, ButtonsType.OK_CANCEL, ok, cancel)
    }

    override fun getInput(
        prompt: String,
        details: String,
        defaultValue: String,
        ok: Consumer<String>,
        cancel: Runnable
    ) = schedule {
        var flags = DialogFlags.DESTROY_WITH_PARENT
        if (dialogParent != null) flags = flags.or(DialogFlags.MODAL)
        val dialog = StringInputDialog(
            dialogParent,
            flags,
            MessageType.QUESTION,
            ButtonsType.OK_CANCEL,
            details,
            defaultValue
        )
        dialog.title = prompt
        dialog.onResponse(processResponses(dialog, { ok.accept(dialog.input) }, cancel))
        dialog.show()
    }

    override fun showLoginRefreshPrompt(account: MicrosoftAccount) =
        schedule { MicrosoftLoginDialog(dialogParent, account).show() }

    private fun simpleDialog(
        markup: String,
        title: String,
        type: MessageType,
        buttons: ButtonsType,
        ok: Runnable?,
        cancel: Runnable?
    ) = schedule { simpleDialog(dialogParent, markup, title, type, buttons, ok, cancel) }

    @JvmStatic
    fun simpleDialog(
        parent: Window?,
        markup: String,
        title: String,
        type: MessageType?,
        buttons: ButtonsType?,
        ok: Runnable?,
        cancel: Runnable?
    ) {
        val dialog =
            MessageDialog(parent, DialogFlags.MODAL.or(DialogFlags.DESTROY_WITH_PARENT), type, buttons, null)
        dialog.title = title
        dialog.markup = markup
        dialog.onResponse(processResponses(dialog, ok, cancel))
        dialog.show()
    }

    private fun processResponses(dialog: Dialog, ok: Runnable?, cancel: Runnable?): Dialog.Response {
        return Dialog.Response { responseId: Int ->
            when (ResponseType.of(responseId)) {
                ResponseType.OK -> {
                    dialog.close()
                    ok?.run()
                }

                ResponseType.CLOSE, ResponseType.CANCEL -> {
                    dialog.close()
                    cancel?.run()
                }

                ResponseType.DELETE_EVENT -> dialog.destroy()
                else -> Utils.LOGGER.error("Unexpected response type: $responseId")
            }
        }
    }
}
